from five import grok

from plone.folder.interfaces import IExplicitOrdering
from plone.uuid.interfaces import IUUID
from zope.component import queryUtility

from seantis.agencies.types import IOrganization
from seantis.agencies.browser.base import BaseView


class OrganizationView(BaseView):

    grok.require('zope2.View')
    grok.context(IOrganization)
    grok.name('view')

    template = grok.PageTemplateFile('templates/organization.pt')

    def suborganizations(self):
        return [
            (brain["Title"] if "Title" in brain else "", brain.getURL())
            for brain in self.context.suborganizations()
        ]

    def memberships(self):
        memberships = []
        for brain in self.context.memberships():
            obj = brain.getObject()
            memberships.append((
                obj.role, obj.person.to_object.title, obj.prefix,
                brain.getURL()
            ))
        return memberships



from seantis.plonetools import setuphandlers

indexes = [
    #('example_index', 'FieldIndex'),
]


def add_catalog_indexes(context, logger=None):
    if indexes:
        setuphandlers.add_catalog_indexes(
            'seantis.agencies', indexes, context, logger
        )


def import_indexes(context):
    if indexes:
        setuphandlers.import_indexes(
            'seantis.agencies', indexes, context
        )



from zope.schema import getFields

from seantis.agencies import tests


class TestSchema(tests.IntegrationTestCase):

    def test_schema_load(self):
        from seantis.agencies.types import IMember, IMembership, IOrganization
        getFields(IMember)
        getFields(IMembership)
        getFields(IOrganization)



# -*- coding: utf-8 -*-

from zope.component import getUtility

from plone import api
from plone.dexterity.interfaces import IDexterityFTI

from seantis.people.supermodel.indexing import update_related_indexes


def run_import_step_from_profile(step, module, profile):
    setup = api.portal.get_tool('portal_setup')
    profile = 'profile-{}:{}'.format(module, profile)
    setup.runImportStepFromProfile(profile, step)


def upgrade_portal_type(portal_type, module, profile):
    run_import_step_from_profile('typeinfo', module, profile)
    update_related_indexes(getUtility(IDexterityFTI, portal_type))


def upgrade_1000_to_1001(context):
    upgrade_portal_type(
        'seantis.agencies.membership', 'seantis.agencies', 'default'
    )

    catalog = api.portal.get_tool('portal_catalog')
    memberships = catalog.unrestrictedSearchResults(
        portal_type='seantis.agencies.membership'
    )

    for membership in [m.getObject() for m in memberships]:
        if not hasattr(membership, 'prefix'):
            membership.prefix = None
        if membership.note == u'*':
            membership.prefix = membership.note
            membership.note = None


def upgrade_1001_to_1002(context):
    upgrade_portal_type(
        'seantis.agencies.membership', 'seantis.agencies', 'default'
    )



import os

from five import grok
from plone import api
from Products.CMFPlone.interfaces import IPloneSiteRoot
from StringIO import StringIO
from xlwt import Workbook


TITLES_REGISTER = [
    u'Akademischer Titel', u'Beruf', u'Vorname', u'Nachname',
    u'Politische Partei', u'Jahrgang', u'E-Mail', u'Adresse', u'Telefon',
    u'Direktnummer', u'Anrede', u'Fax', u'Website', u'Stichworte',
    u'Bemerkungen', u'Organisationen'
]
FIELDS_REGISTER = [
    'academic_title', 'occupation', 'firstname', 'lastname',
    'political_party', 'year', 'email', 'address', 'phone', 'direct_number',
    'salutation', 'fax', 'website', 'keywords', 'remarks'
]

TITLES_ORGANIZATION = [
    u'ID', u'Unterorganisationen', u'Titel', u'Beschreibung', u'Portrait'
]
FIELDS_ORGANIZATION = [
    'title', 'description', 'portrait'
]


class ExportView(grok.View):

    grok.name('export-agencies')
    grok.context(IPloneSiteRoot)
    grok.require('cmf.ManagePortal')

    template = None

    def get_root_organizations(self):
        catalog = api.portal.get_tool('portal_catalog')
        folder_path = '/'.join(self.context.getPhysicalPath())
        organizations = catalog(
            path={'query': folder_path, 'depth': 1},
            portal_type='seantis.agencies.organization',
            sort_on='getObjPositionInParent',
        )
        return [organization.getObject() for organization in organizations]

    def get_root_registers(self):
        catalog = api.portal.get_tool('portal_catalog')
        folder_path = '/'.join(self.context.getPhysicalPath())
        organizations = catalog(
            path={'query': folder_path, 'depth': 1},
            portal_type='seantis.people.list',
            sort_on='getObjPositionInParent',
        )
        return [organization.getObject() for organization in organizations]

    def get_simple_uid(self, uid, uids):
        if uid not in uids:
            uids[uid] = uids['next']
            uids['next'] = uids['next'] + 1
        return uids[uid]

    def write_organization(self, sheet, organization, uids):
        uid = self.get_simple_uid(organization.UID(), uids)
        children = [o.getObject() for o in organization.suborganizations()]
        memberships = [m.getObject() for m in organization.memberships()]

        index = len(sheet.rows)
        sheet.row(index).write(0, str(uid))
        sheet.row(index).write(1, ','.join(
            [str(self.get_simple_uid(child.UID(), uids)) for child in children]
        ))
        for col, field in enumerate(FIELDS_ORGANIZATION):
            sheet.row(index).write(2+col, getattr(organization, field))

        for child in children:
            self.write_organization(sheet, child, uids)

    def write_register(self, sheet, register, uids):
        catalog = api.portal.get_tool('portal_catalog')
        folder_path = '/'.join(register.getPhysicalPath())

        people = catalog(
            path={'query': folder_path, 'depth': 1},
            portal_type='seantis.agencies.member'
        )
        for index, brain in enumerate(people):
            person = brain.getObject()

            for col, field in enumerate(FIELDS_REGISTER):
                sheet.row(index+1).write(col, getattr(person, field))

            memberships = [
                u'(%s)(%s)(%s)(%s)(%s)' % (
                    str(uids.get(item.aq_parent.UID(), '')),
                    item.role, item.start or '', item.prefix or '',
                    str(item.aq_parent.getObjectPosition(item.getId()))
                )
                for sublist in person.memberships.values() for item in sublist
            ]

            sheet.row(index+1).write(
                len(FIELDS_REGISTER), u', '.join(memberships)
            )

    def render(self):
        book = Workbook(encoding='utf-8')
        uids = {'next': 0}

        root_organizations = self.get_root_organizations()
        for organization in root_organizations:
            sheet = book.add_sheet(organization.title)
            for index, cell in enumerate(TITLES_ORGANIZATION):
                sheet.row(0).write(index, cell)
            self.write_organization(sheet, organization, uids)

        root_registers = self.get_root_registers()
        for register in root_registers:
            sheet = book.add_sheet(register.title)
            for index, cell in enumerate(TITLES_REGISTER):
                sheet.row(0).write(index, cell)

            self.write_register(sheet, register, uids)

        filehandle = StringIO()
        if root_organizations or root_registers:
            book.save(filehandle)
        response = filehandle.getvalue()

        filehandle.seek(0, os.SEEK_END)
        filesize = filehandle.tell()
        filehandle.close()

        self.request.RESPONSE.setHeader('Content-disposition', 'export.xls')
        self.request.RESPONSE.setHeader('Content-Type', 'application/xls')
        self.request.RESPONSE.setHeader('Content-Length', filesize)

        return response



import logging
log = logging.getLogger('seantis.agencies')

import re
import transaction
import xlrd

from five import grok
from plone.dexterity.utils import createContentInContainer
from plone.directives import form
from plone.namedfile.field import NamedFile
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from StringIO import StringIO
from z3c.form import field
from z3c.form.button import buttonAndHandler
from z3c.form.error import InvalidErrorViewSnippet
from z3c.relationfield.relation import RelationValue
from zope.app.intid.interfaces import IIntIds
from zope.component import getUtility
from zope.interface import Invalid

from seantis.agencies import _
from seantis.agencies.browser.xlsexport import (
    TITLES_REGISTER, FIELDS_REGISTER,
    TITLES_ORGANIZATION, FIELDS_ORGANIZATION
)
from seantis.plonetools import unrestricted


KNOWN_PUBLISH_ACTIONS = (
    'publish', 'onegov-simple-workflow--TRANSITION--publish--draft_published'
)


class IImportSchema(form.Schema):
    xls_file = NamedFile(title=_(u"XLS file"))


class ImportView(form.Form):
    fields = field.Fields(IImportSchema)

    grok.context(IPloneSiteRoot)
    grok.require('cmf.ManagePortal')
    grok.name('import-agencies')

    ignoreContext = True

    def abort_action(self, action, messages):
        """ Aborts the given action and adds the list of messages as
        error-widgets to the form."""
        form = action.form
        formcontent = form.getContent()
        request = action.request

        for msg in messages:
            args = (Invalid(msg), request, None, None, form, formcontent)
            err = InvalidErrorViewSnippet(*args)
            err.update()
            form.widgets.errors += (err, )

        form.status = form.formErrorsMessage

        transaction.abort()

    @buttonAndHandler(_(u'Import'), name='import')
    def import_xls(self, action):
        """ Create and handle form button."""

        # Extract form field values and errors from HTTP request
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        try:
            io = StringIO(data["xls_file"].data)
            workbook = xlrd.open_workbook(file_contents=io.read())
        except (KeyError, TypeError, xlrd.XLRDError):
            self.abort_action(action, (_(u'Invalid XLS file'),))
            return

        errors = self._import_xls(workbook)

        if errors:
            self.abort_action(action, errors)
        else:
            self.status = _(u'Items imported. Please reindex.')

    def parse_organization_sheet(self, sheet, errors):
        organizations = {}
        for row in range(1, sheet.nrows):
            try:
                values = [cell.value.strip() for cell in sheet.row(row)]
                assert len(values) == len(TITLES_ORGANIZATION)
                assert values[0] not in organizations

                organization = {}
                for index, field in enumerate(FIELDS_ORGANIZATION):
                    organization[field] = values[2+index]

                children = [id.strip() for id in values[1].split(',') if id]
                organization['_children'] = children

                organizations[values[0]] = organization
            except:
                errors.append(_(u'Invalid row ${row} in sheet ${sheet}',
                                mapping={u'row': row, u'sheet': sheet.name}))

        found = 0
        start = None
        for key, value in organizations.iteritems():
            if sheet.name == value['title']:
                start = key
                found += 1
        if found != 1:
            errors.append(_(u'Top level organizations not found'))
        else:
            organizations['first'] = start

        return organizations

    def parse_register_sheet(self, sheet, errors):
        members = []

        for row in range(1, sheet.nrows):
            try:
                values = [cell.value.strip() for cell in sheet.row(row)]
                assert len(values) == len(TITLES_REGISTER)

                member = {}
                for index, field in enumerate(FIELDS_REGISTER):
                    member[field] = values[index]

                pattern = re.compile(
                    r'\((.*)\)\((.*)\)\((.*)\)\((.*)\)\((.*)\)'
                )
                strings = [
                    membership.strip() for membership
                    in values[len(FIELDS_REGISTER)].split(',') if membership
                ]
                memberships = [pattern.match(str).groups() for str in strings]
                member['_memberships'] = memberships

                members.append(member)
            except:
                errors.append(_(u'Invalid row ${row} in sheet ${sheet}',
                                mapping={u'row': row, u'sheet': sheet.name}))

        return members

    def publish_content(self, content):
        wftool = getToolByName(self.context, 'portal_workflow')
        with unrestricted.run_as('Manager'):
            for action in KNOWN_PUBLISH_ACTIONS:
                try:
                    wftool.doActionFor(content, action)
                    break
                except WorkflowException:
                    pass

    def create_organisation(self, context, id, organizations, memberships,
                            count=0):
        organization = organizations[id]

        kwargs = dict(
            (key, unicode(value)) for (key, value) in organization.iteritems()
            if not key.startswith('_')
        )
        content = createContentInContainer(
            context, "seantis.agencies.organization", **kwargs
        )
        self.publish_content(content)
        count += 1
        log.info(
            'added organization %s (%i/%i)' % (
                organization['title'], count, len(organizations)-1
            )
        )

        if id in memberships:
            for membership in sorted(memberships[id], key=lambda i: i[3]):
                person = RelationValue(membership[4])
                m_content = createContentInContainer(
                    content, "seantis.agencies.membership",
                    role=unicode(membership[0]),
                    start=unicode(membership[1]),
                    prefix=unicode(membership[2]),
                    person=person
                )
                self.publish_content(m_content)
                log.info('added %s to %s' % (person.to_object.title,
                                             organization['title']))

        for child in organization['_children']:
            count = self.create_organisation(
                content, child, organizations, memberships, count
            )

        return count

    def _import_xls(self, workbook):
        errors = []
        organizations = {}
        registers = {}

        for sheet in workbook.sheets():
            titles = []
            if sheet.nrows > 1:
                titles = [cell.value for cell in sheet.row(0)]

            if titles == TITLES_ORGANIZATION:
                organizations[sheet.name] = self.parse_organization_sheet(
                    sheet, errors
                )
            elif titles == TITLES_REGISTER:
                registers[sheet.name] = self.parse_register_sheet(
                    sheet, errors
                )
            else:
                log.warn('ignored sheet %s' % sheet.name)

        if errors:
            return errors

        intids = getUtility(IIntIds)
        memberships = {}
        for name, members in registers.iteritems():
            directory = createContentInContainer(
                self.context, "seantis.people.list", title=name
            )
            self.publish_content(directory)
            log.info('added directory %s' % name)

            for index, member in enumerate(members):
                kwargs = dict(
                    (key, unicode(value)) for (key, value)
                    in member.iteritems() if not key.startswith('_')
                )
                content = createContentInContainer(
                    directory, "seantis.agencies.member",
                    exclude_from_nav=True, **kwargs
                )
                self.publish_content(content)
                uid = intids.getId(content)
                log.info(
                    'added person %s %s (%i/%i)' % (
                        member['firstname'], member['lastname'], index+1,
                        len(members)
                    )
                )

                for membership in member['_memberships']:
                    if membership[0] not in memberships:
                        memberships[membership[0]] = []
                    memberships[membership[0]].append((
                        membership[1], membership[2], membership[3],
                        membership[4], uid
                    ))

        for name, organization in organizations.iteritems():
            self.create_organisation(
                self.context, organization['first'], organization, memberships
            )

        log.info('import finished')



from .base import BaseView, BaseForm
from .renderer import UUIDListRenderer
from seantis.people.browser.renderer import renderers
from seantis.people.utils import UUIDList


# Patch the renderer to include the parent organization
renderers[UUIDList] = UUIDListRenderer()



import logging
log = logging.getLogger('seantis.agencies')

import os
import codecs
import re
import tempfile

from datetime import datetime, date
from pdfdocument.document import MarkupParagraph
from reportlab.lib.units import cm

from five import grok
from plone import api
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zExceptions import NotFound
from zope.interface import Interface

from kantonzugpdf import ReportZug

from seantis.agencies import _
from seantis.plonetools import tools


class OrganizationsReport(ReportZug):
    """ Report to show the memberships of organizations found in the
    current context.

    """

    def translate(self, text):
        return tools.translator(self.request, 'seantis.agencies')(text)

    def get_print_date_text(self):
        date_text = api.portal.get_localized_time(
            datetime=datetime.combine(
                date.today(), datetime.min.time()
            )
        )

        return self.translate(
            _(u'Print date: ${date}', mapping={'date': date_text})
        )

    def populate(self):
        """ Builds the structure of the report before it gets rendered. """
        self.title = self.context.title
        self.adjust_style()

        # First page contains the title and table of contents
        self.pdf.h(self.title)
        self.pdf.table_of_contents()
        self.pdf.pagebreak()

        # Iterate recursive over organizations
        root_organizations = self.get_root_organizations()

        for organization in root_organizations:
            self.populate_organization(organization, 0)

    def get_root_organizations(self):
        catalog = api.portal.get_tool('portal_catalog')
        folder_path = '/'.join(self.context.getPhysicalPath())
        organizations = catalog(
            path={'query': folder_path, 'depth': 1},
            portal_type='seantis.agencies.organization',
            sort_on='getObjPositionInParent',
        )
        return [organization.getObject() for organization in organizations]

    def populate_organization(self, organization, level):

        has_content = False
        if level:
            # Title
            if level == 1:
                self.pdf.h1(organization.title)
            elif level == 2:
                self.pdf.h2(organization.title)
            elif level == 3:
                self.pdf.h3(organization.title)
            elif level == 4:
                self.pdf.h4(organization.title)
            elif level == 5:
                self.pdf.h5(organization.title)
            elif level == 6:
                self.pdf.h6(organization.title)
            else:
                self.pdf.p(organization.title)

            # Portrait
            if organization.portrait.strip():
                has_content = True
                self.pdf.spacer()
                try:
                    # remove target attribute from link tags
                    self.pdf.p_markup(
                        re.sub(r"target=[\"'].*\w[\"']", "",
                               organization.portrait)
                    )
                except ValueError:
                    # the portrait might have some unprocessable html code
                    self.pdf.p(organization.portrait)
                    log.warn('%s contains invalid markup' % organization.title)

            # Table with memberships
            table_data = []
            memberships = [m.getObject() for m in organization.memberships()]
            for membership in memberships:
                person = membership.person.to_object
                fields = ['title', 'year', 'academic_title', 'occupation',
                          'address', 'political_party']
                text = ', '.join([
                    getattr(person, field) for field in fields
                    if getattr(person, field)
                ])

                table_data.append([
                    MarkupParagraph(membership.role, self.pdf.style.normal),
                    MarkupParagraph(membership.prefix, self.pdf.style.normal),
                    MarkupParagraph(text, self.pdf.style.normal),
                ])

            table_columns = [4.5*cm, 0.3*cm, 11*cm]
            if table_data:
                has_content = True
                self.pdf.spacer()
                self.pdf.table(table_data, table_columns)

        if has_content:
            self.pdf.pagebreak()
        else:
            self.pdf.spacer()

        children = [o.getObject() for o in organization.suborganizations()]
        for child in children:
            self.populate_organization(child, level+1)


class PdfExportView(grok.View):

    grok.name('pdfexport-agencies')
    grok.context(IPloneSiteRoot)
    grok.require('zope2.View')

    template = None

    def render(self):
        filename = _(u'Organizations')
        filehandle = OrganizationsReport().build(self.context, self.request)

        filename = codecs.utf_8_encode('filename="%s.pdf"' % filename)[0]
        self.request.RESPONSE.setHeader('Content-disposition', filename)
        self.request.RESPONSE.setHeader('Content-Type', 'application/pdf')

        response = filehandle.getvalue()
        filehandle.seek(0, os.SEEK_END)

        filesize = filehandle.tell()
        filehandle.close()

        self.request.RESPONSE.setHeader('Content-Length', filesize)

        return response



from Acquisition import aq_inner, aq_parent

from plone.app.uuid.utils import uuidToCatalogBrain

from seantis.people.browser.renderer import (
    UUIDListRenderer as BaseUUIDListRenderer
)
from seantis.plonetools import tools


class UUIDListRenderer(BaseUUIDListRenderer):

    def brain_to_link(self, brain):
        obj = brain.getObject()
        parent = aq_parent(aq_inner(obj))

        title = u'%s' % obj.title
        if parent:
            # title = u'%s > %s' % (parent.title, title)
            title = u'%s (%s)' % (title, parent.title)

        return (brain.getURL(), title)

    def __call__(self, context, field, options):

        if not all([field == 'organization_uuids',
                    context.portal_type == 'seantis.agencies.member']):
            return super(UUIDListRenderer, self).__call__(context, field,
                                                          options)

        uuids = getattr(context, field, None)
        if not uuids:
            return u''

        brains = (b for b in (uuidToCatalogBrain(uid) for uid in uuids) if b)
        items = (self.brain_to_link(b) for b in brains)

        unicode_sortkey = tools.unicode_collate_sortkey()
        items = sorted(items, key=lambda i: unicode_sortkey(i[1]))

        return ', '.join(
            self.template.substitute(url=url, title=title)
            for url, title in items
        )



from plone import api

from zope.security.management import newInteraction, endInteraction

from seantis.plonetools.testing import TestCase
from seantis.agencies.testing import (
    INTEGRATION_TESTING,
    FUNCTIONAL_TESTING
)


# to use with integration where security interactions need to be done manually
class IntegrationTestCase(TestCase):
    layer = INTEGRATION_TESTING

    def setUp(self):
        super(IntegrationTestCase, self).setUp()
        newInteraction()

    def tearDown(self):
        endInteraction()
        super(IntegrationTestCase, self).tearDown()


# to use with the browser which does its own security interactions
class FunctionalTestCase(TestCase):
    layer = FUNCTIONAL_TESTING


class BrowserTestCase(FunctionalTestCase):

    def setUp(self):
        super(BrowserTestCase, self).setUp()
        self.baseurl = self.portal.absolute_url()
        self.admin_browser = browser = self.new_admin_browser()

        browser.open(self.baseurl + '/createObject?type_name=Folder')
        browser.getControl('Title').value = 'testfolder'
        browser.getControl('Save').click()

        self.folder_url = self.baseurl + '/testfolder'
        self.infolder = lambda url: self.folder_url + url

    def tearDown(self):
        self.admin_browser.open(self.infolder('/delete_confirmation'))
        self.admin_browser.getControl('Delete').click()

        super(BrowserTestCase, self).tearDown()

    def new_admin_browser(self):
        browser = self.new_browser()
        browser.login_admin()

        return browser



from five import grok

from plone.folder.interfaces import IExplicitOrdering
from plone.uuid.interfaces import IUUID
from zope.component import queryUtility

from seantis.agencies.types import IMembership
from seantis.agencies.browser.base import BaseView


class OrganizationView(BaseView):

    grok.require('zope2.View')
    grok.context(IMembership)
    grok.name('view')

    template = grok.PageTemplateFile('templates/membership.pt')

    def person(self):
        person = self.context.person.to_object
        return [person.title, person.absolute_url()]



from five import grok

from seantis.plonetools.browser import BaseView as SharedBaseView
from seantis.plonetools.browser import BaseForm as SharedBaseForm

from seantis.agencies.interfaces import ISeantisAgenciesSpecific


class BaseView(SharedBaseView):

    grok.baseclass()
    grok.layer(ISeantisAgenciesSpecific)

    domain = 'seantis.agencies'


class BaseForm(SharedBaseForm):

    grok.baseclass()
    grok.layer(ISeantisAgenciesSpecific)

    domain = 'seantis.agencies'



import logging
log = logging.getLogger('seantis.agencies')

from datetime import date
from five import grok

from zope import schema
from zope.interface import Interface, Invalid

from plone import api
from plone.app.z3cform.wysiwyg import WysiwygFieldWidget
from plone.dexterity.content import Container
from plone.directives import form
from plone.app.dexterity.behaviors.exclfromnav import IExcludeFromNavigation

from collective.dexteritytextindexer import searchable

from seantis.people.types.base import PersonBase
from seantis.people.interfaces import IMembership as IPeopleMembership
from seantis.people.content import Membership as PeopleMembership
from seantis.agencies import _


class IMember(form.Schema):
    form.model("member.xml")


class Member(PersonBase):
    pass


@form.default_value(field=IExcludeFromNavigation['exclude_from_nav'])
def excludeFromNavDefaultValue(data):
    return True


class IOrganization(form.Schema):

    searchable('title')
    title = schema.TextLine(
        title=_(u'Title')
    )

    searchable('description')
    description = schema.Text(
        title=_(u'Description'),
        required=False
    )

    searchable('portrait')
    form.widget(portrait=WysiwygFieldWidget)
    portrait = schema.Text(
        title=_(u'Portrait'),
        required=False
    )


class Organization(Container):

    def memberships(self):
        catalog = api.portal.get_tool('portal_catalog')
        folder_path = '/'.join(self.getPhysicalPath())

        memberships = catalog(
            path={'query': folder_path, 'depth': 1},
            portal_type='seantis.agencies.membership',
            sort_on='getObjPositionInParent',
        )

        return memberships

    def suborganizations(self):
        catalog = api.portal.get_tool('portal_catalog')
        folder_path = '/'.join(self.getPhysicalPath())

        suborganizations = catalog(
            path={'query': folder_path, 'depth': 1},
            portal_type='seantis.agencies.organization',
            sort_on='getObjPositionInParent',
        )

        return suborganizations


class IMembership(IPeopleMembership):

    start = schema.TextLine(
        title=_(u"Start of membership"),
        required=False
    )

    prefix = schema.TextLine(
        title=_(u"Prefix"),
        required=False
    )


class Membership(PeopleMembership):
    pass



from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting
from plone.app.testing import applyProfile
from plone.app.testing import quickInstallProduct

from plone.testing import z2
from Testing import ZopeTestCase
from zope.configuration import xmlconfig


class TestLayer(PloneSandboxLayer):

    default_bases = (PLONE_FIXTURE,)

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def setUpZope(self, app, configurationContext):

        # Set up sessioning objects
        app.REQUEST['SESSION'] = self.Session()
        ZopeTestCase.utils.setupCoreSessions(app)

        import seantis.plonetools
        self.loadZCML(package=seantis.plonetools)

        import seantis.agencies
        self.loadZCML(package=seantis.agencies)

        xmlconfig.file(
            'configure.zcml',
            seantis.agencies,
            context=configurationContext
        )

    def setUpPloneSite(self, portal):
        quickInstallProduct(portal, 'seantis.agencies')
        applyProfile(portal, 'seantis.agencies:default')

    def tearDownZope(self, app):
        z2.uninstallProduct(app, 'seantis.agencies')


Layer = TestLayer()


INTEGRATION_TESTING = IntegrationTesting(
    bases=(Layer, ),
    name="Testfixture:Integration"
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(Layer, ),
    name="Testfixture:Functional"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages

name = 'seantis.agencies'
description = (
    'A directory of people for (government) agencies.'
)
version = '0.2'

requirements = [
    'Plone>=4.3',
    'plone.api',
    'plone.app.dexterity [grok]',
    'five.grok',
    'xlrd',
    'xlwt',
    'seantis.plonetools',
    'seantis.people>=0.24',
    'kantonzugpdf'
]


test_requirements = [
    'plone.app.testing',
    'collective.betterbrowser[pyquery]',
    'seantis.plonetools[tests]',
    'mock'
]


def get_long_description():
    readme = open('README.rst').read()
    history = open(os.path.join('docs', 'HISTORY.rst')).read()

    # cut the part before the description to avoid repetition on pypi
    readme = readme[readme.index(description) + len(description):]

    return '\n'.join((readme, history))

setup(
    name=name, version=version, description=description,
    long_description=get_long_description(),
    classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='seantis.agencies plone',
    author='Seantis GmbH',
    author_email='Seantis GmbH',
    url='https://github.com/seantis/seantis.agencies',
    license='GPLv2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages='seantis.agencies'.split('.')[:-1],
    include_package_data=True,
    zip_safe=False,
    install_requires=requirements,
    extras_require=dict(
        tests=test_requirements
    ),
    entry_points="""
    # -*- Entry points: -*-

    [z3c.autoinclude.plugin]
    target = plone
    """
)



import xlrd

from copy import deepcopy
from datetime import date, timedelta
from plone import api
from StringIO import StringIO
from xlwt import Workbook
from zExceptions import Unauthorized

from seantis.agencies import tests
from seantis.agencies.browser.xlsexport import TITLES_ORGANIZATION
from seantis.agencies.browser.xlsexport import TITLES_REGISTER
from seantis.plonetools import tools


class TestImportExport(tests.BrowserTestCase):

    def setUp(self):
        super(TestImportExport, self).setUp()
        self.organizations = [
            [u'0', u'1,2', u'Organizations', u'des_0', u'port_0'],
            [u'1', u'', u'org_1', u'des_1', u'port_1'],
            [u'2', u'3', u'org_2', u'des_2', u'port_2'],
            [u'3', u'', u'org_3', u'des_3', u'port_3']
        ]
        self.people = [
            [str(prefix) + '_' + field for field in TITLES_REGISTER]
            for prefix in range(3)
        ]
        self.people[0][-1] = u'(0)(role)(2006)(*)(3)'
        self.people[1][-1] = u'(1)()()()(7), (0)()()()(2),(3)()()()(7),'
        self.people[2][-1] = u'(2)(role)()(2000)(2),(3)()(*)()(8)'

    def create_test_workbook(self, filehandle, organizations=None,
                             people=None):
        if organizations is None:
            organizations = self.organizations
        if people is None:
            people = self.people

        book = Workbook(encoding='utf-8')
        sheet = book.add_sheet(u'Organizations')
        for index, cell in enumerate(TITLES_ORGANIZATION):
            sheet.row(0).write(index, cell)
        for row, organization in enumerate(organizations):
            for index, cell in enumerate(organization):
                sheet.row(row+1).write(index, cell)
        sheet = book.add_sheet(u'People')
        for index, cell in enumerate(TITLES_REGISTER):
            sheet.row(0).write(index, cell)
        for row, organization in enumerate(people):
            for index, cell in enumerate(organization):
                sheet.row(row+1).write(index, cell)
        book.save(filehandle)

    def test_access(self):
        browser = self.new_browser()
        self.assertRaises(Unauthorized, browser.open, '/export-agencies')
        self.assertRaises(Unauthorized, browser.open, '/import-agencies')

    def test_export_empty(self):
        browser = self.new_admin_browser()
        browser.open('/export-agencies')
        self.assertEquals(browser.contents, '')
        self.assertEquals(browser.headers['Content-disposition'], 'export.xls')
        self.assertEquals(browser.headers['Content-Type'], 'application/xls')
        self.assertEquals(browser.headers['Content-Length'], '0')

    def test_import_export(self):
        filehandle = StringIO()
        self.create_test_workbook(filehandle)
        xls_file = filehandle.getvalue()
        filehandle.close()

        # import
        browser = self.new_admin_browser()
        browser.open('/import-agencies')
        widget = browser.getControl(name='form.widgets.xls_file')
        widget.add_file(xls_file, 'application/xls', 'import.xls')
        browser.getControl('Import').click()

        # export
        browser.open('/export-agencies')
        self.assertEquals(browser.headers['Content-disposition'], 'export.xls')
        self.assertEquals(browser.headers['Content-Type'], 'application/xls')
        self.assertTrue(int(browser.headers['Content-Length']) > 0)

        # compare
        workbook = xlrd.open_workbook(file_contents=browser.contents)
        sheets = workbook.sheets()
        self.assertEquals(len(sheets), 2)
        self.assertEquals(sheets[0].name, u'Organizations')
        self.assertEquals(sheets[0].ncols, len(TITLES_ORGANIZATION))
        self.assertEquals(sheets[0].nrows, 5)
        for row, content in enumerate(self.organizations):
            self.assertEquals([cell.value for cell in sheets[0].row(row+1)],
                              content)

        self.assertEquals(sheets[1].name, u'People')
        self.assertEquals(sheets[1].ncols, len(TITLES_REGISTER))
        self.assertEquals(sheets[1].nrows, 4)
        for row, content in enumerate(self.people):
            self.assertEquals(
                [cell.value for cell in sheets[1].row(row+1)[:-1]],
                content[:-1]
            )

        # roles are slightly different: position starts at zero, order is
        # random
        self.assertEquals(u'(0)(role)(2006)(*)(1)',
                          sheets[1].cell(1, -1).value)
        self.assertTrue(u'(1)()()()(0)', sheets[1].cell(2, -1).value)
        self.assertTrue(u'(3)()()()(0)', sheets[1].cell(2, -1).value)
        self.assertTrue(u'(0)()()()(0)', sheets[1].cell(2, -1).value)
        self.assertTrue(u'(2)(role)()()(0)', sheets[1].cell(3, -1).value)
        self.assertTrue(u'(3)()()(*)(1)', sheets[1].cell(3, -1).value)

        # browse imported items
        browser.open('/organizations')
        self.assertIn('des_0', browser.contents)
        self.assertIn('port_0', browser.contents)
        self.assertIn('org_1', browser.contents)
        self.assertIn('org_2', browser.contents)
        self.assertNotIn('org_3', browser.contents)

        browser.open('/organizations/org_1')
        self.assertIn('des_1', browser.contents)
        self.assertIn('port_1', browser.contents)
        self.assertNotIn('org_2', browser.contents)
        self.assertNotIn('org_3', browser.contents)

        browser.open('/organizations/org_2')
        self.assertIn('des_2', browser.contents)
        self.assertIn('port_2', browser.contents)
        self.assertNotIn('org_1', browser.contents)
        self.assertIn('org_3', browser.contents)

        browser.open('/organizations/org_2/org_3')
        self.assertIn('des_3', browser.contents)
        self.assertIn('port_3', browser.contents)

    def test_import_errors(self):
        browser = self.new_admin_browser()

        # missing file
        browser.open('/import-agencies')
        browser.getControl('Import').click()
        self.assertIn('Required input is missing', browser.contents)

        # invalid file
        browser.open('/import-agencies')
        widget = browser.getControl(name='form.widgets.xls_file')
        widget.add_file('stuff', 'application/xls', 'import.xls')
        browser.getControl('Import').click()
        self.assertIn('Invalid XLS file', browser.contents)

        # error in register sheet
        people = deepcopy(self.people)
        people[1][-1] = u'(1))()()(7), (0aa()()(2),(3)()()()(7),'
        filehandle = StringIO()
        self.create_test_workbook(filehandle, people=people)
        xls_file = filehandle.getvalue()
        filehandle.close()

        browser.open('/import-agencies')
        widget = browser.getControl(name='form.widgets.xls_file')
        widget.add_file(xls_file, 'application/xls', 'import.xls')
        browser.getControl('Import').click()
        self.assertIn('Invalid row 2 in sheet People', browser.contents)

        # error in organization sheet
        organizations = deepcopy(self.organizations)
        organizations[0][2] = u'Urganizations'
        organizations[2][0] = u'1'
        filehandle = StringIO()
        self.create_test_workbook(filehandle, organizations=organizations)
        xls_file = filehandle.getvalue()
        filehandle.close()

        browser.open('/import-agencies')
        widget = browser.getControl(name='form.widgets.xls_file')
        widget.add_file(xls_file, 'application/xls', 'import.xls')
        browser.getControl('Import').click()
        self.assertIn('Top level organizations not found', browser.contents)
        self.assertIn('Invalid row 3 in sheet Organizations', browser.contents)
